@@ -234,7 +234,7 @@ sub _stage2 {
                             {type => 'GROUP',
                              start => $textnode2->{start} - 1,
                              end => $textnode2->{end} + 1,
-                             children => LaTeX::TOM::Tree->new([$textnode2], $parser)
+                             children => LaTeX::TOM::Tree->new([$textnode2], $parser),
                             });
 
                         # splice the new subtree into the old location
@@ -271,10 +271,11 @@ sub _stage2 {
                              start => $textnode2->{start} - 1,
                              end => $textnode3->{end} + 1,
                              children => LaTeX::TOM::Tree->new(
-                                [$textnode2, 
+                                [$textnode2,
                                  @removed,
-                                 $textnode3]),
-                                 $parser});
+                                 $textnode3],
+                                 $parser),
+                            });
 
                         # replace the two original text nodes with the leftover left and
                         # right portions, as well as the group node with everything in
@@ -357,7 +358,7 @@ sub _stage3 {
             # due to the previous parsing stages. if found, the parent node is
             # promoted to a command.
             #
-            if ($text =~ /^\s*\\(\w+)/ && defined $parent && defined $parser->{config}{INNERCMDS}->{$1}) {
+            if ($text =~ /^\s*\\(\w+\*?)/ && defined $parent && defined $parser->{config}{INNERCMDS}->{$1}) {
                 my $command = $1;
 
                 # if the parent is already a command node, we have to make a new
@@ -397,14 +398,14 @@ sub _stage3 {
                     $parser->{USED_COMMANDS}->{$parent->{command}} = 1;
                 }
 
-                $node->{content} =~ s/^\s*\\(?:\w+)//o;
+                $node->{content} =~ s/^\s*\\(?:\w+\*?)//o;
             }
 
             # outer command (such as \command{parameters}). our regexp checks to
             # see if this text chunk ends in \command, since that would be the case
             # due to the previous parsing stages.
             #
-            if ($text =~ /(?:^|[^\\])(\\\w+(\s*\[.*?\])?)\s*$/os && 
+            if ($text =~ /(?:^|[^\\])(\\\w+\*?(\s*\[.*?\])?)\s*$/os && 
                     defined $tree->{nodes}[$i+1] &&
                     $tree->{nodes}[$i+1]->{type} eq 'GROUP') {
 
@@ -413,10 +414,10 @@ sub _stage3 {
                 #print "found text node [$text] with command tag [$tag]\n";
 
                 # remove the text
-                $node->{content} =~ s/\\\w+\s*(?:\[.*?\])?\s*$//os;
+                $node->{content} =~ s/\\\w+\*?\s*(?:\[.*?\])?\s*$//os;
 
                 # parse it for command and ops
-                $tag =~ /^\\(\w+)\s*(?:\[(.*?)\])?$/os;
+                $tag =~ /^\\(\w+\*?)\s*(?:\[(.*?)\])?$/os;
 
                 my $command = $1;
                 my $opts = $2;
@@ -434,7 +435,7 @@ sub _stage3 {
 
             # recognize braceless commands
             #
-            if ($text =~ /(\\(\w+)[ \t]+(\S+))/gso || $text =~ /(\\(\w+)(\d+))/gso) {
+            if ($text =~ /(\\(\w+\*?)[ \t]+(\S+))/gso || $text =~ /(\\(\w+)(\d+))/gso) {
                 my $all = $1;
                 my $command = $2;
                 my $param = $3;
@@ -466,7 +467,8 @@ sub _stage3 {
                          command => $command,
                          start => $node->{start} + $a,
                          end => $node->{start} + $b,
-                         children => LaTeX::TOM::Tree->new([$newchild]), $parser });
+                         children => LaTeX::TOM::Tree->new([$newchild], $parser),
+                        });
 
                     $parser->{USED_COMMANDS}->{$commandnode->{command}} = 1;
 
@@ -563,7 +565,8 @@ sub _stage4 {
                      end => $end,
                      ostart => $start - length('begin') - length($class) - 2,
                      oend => $end + length('end') + length($class) + 2,
-                     children => LaTeX::TOM::Tree->new([@newarray]), $parser });
+                     children => LaTeX::TOM::Tree->new([@newarray], $parser),
+                    });
 
                 if (defined $parser->{config}{MATHENVS}->{$envnode->{class}}) {
                     $envnode->{math} = 1;
@@ -681,7 +684,8 @@ sub _stage5_r {
                                 ostart => $startpos + $leftpos - length($left) + 1,
                                 end => $startpos + $rightpos,
                                 oend => $startpos + $rightpos + length($right) - 1,
-                                children => LaTeX::TOM::Tree->new([$textnode2]), $parser });
+                                children => LaTeX::TOM::Tree->new([$textnode2], $parser),
+                                });
 
                             splice @{$tree->{nodes}}, $i, 1, $textnode1, $mathnode, $textnode3;
 
@@ -719,7 +723,8 @@ sub _stage5_r {
                                 children => LaTeX::TOM::Tree->new(
                                 [$textnode2,
                                  @remnodes,
-                                 $textnode3]), $parser });
+                                 $textnode3], $parser),
+                                });
 
                             # replace (TEXT_A, ... , TEXT_B) with the mathnode created above
                             splice @{$tree->{nodes}}, $leftidx, 2, $textnode1, $mathnode, $textnode4;
@@ -807,9 +812,11 @@ sub _propegateModes {
             # handle math or plain text forcing envs
             #
             if ($node->{type} eq 'ENVIRONMENT' || $node->{type} eq 'COMMAND') {
-                if (defined $parser->{config}{MATHENVS}->{$node->{class}} || 
-                    defined $parser->{config}{MATHENVS}->{"$node->{class}*"}) {
-
+                if (defined $node->{class} && (
+                      defined $parser->{config}{MATHENVS}->{$node->{class}} ||
+                      defined $parser->{config}{MATHENVS}->{"$node->{class}*"})
+                   )
+                {
                     $mathflag = 1;
                     $plaintextflag = 0;
                 }
@@ -875,7 +882,7 @@ sub _applyMapping {
         #
         if ($node->{type} eq 'COMMAND' &&
                 $node->{command} eq 'begin' &&
-                $node->{children}->[0]->{nodes}{content} eq $mapping->{name}) {
+                $node->{children}->{nodes}[0]->{content} eq $mapping->{name}) {
 
             # grab the nparams next group nodes as parameters
             #
@@ -1351,7 +1358,7 @@ sub _findbrace {
         $realbrace = 1;
         $index_c = index $text, '}', $pos_c;
 
-        if (substr($text, $index_c - 1, 1) eq ' ') {
+        if (($index_c - 1) >= 0 && substr($text, $index_c - 1, 1) eq ' ') {
             $pos_c = $index_c + 1;
             $index_c = -1;
         }
